﻿using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;
using Ivi.Visa.Interop;
//'' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
//''  © Agilent Technologies, Inc. 2013
//''
//'' You have a royalty-free right to use, modify, reproduce and distribute
//'' the Sample Application Files (and/or any modified version) in any way
//'' you find useful, provided that you agree that Agilent Technologies has no
//'' warranty,  obligations or liability for any Sample Application Files.
//''
//'' Agilent Technologies provides programming examples for illustration only,
//'' This sample program assumes that you are familiar with the programming
//'' language being demonstrated and the tools used to create and debug
//'' procedures. Agilent Technologies support engineers can help explain the
//'' functionality of Agilent Technologies software components and associated
//'' commands, but they will not modify these samples to provide added
//'' functionality or construct procedures to meet your specific needs.
//'' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

namespace FastestDCVMeasurements
{
    class FastDCV
    {
        static void Main(string[] args)
        {
            FastDCV DmmClass = new FastDCV(); //Create an instance of this class so we can call functions from Main

            /*In this example program we try to configure the DMM for the fastest DCV readings. To do this, we turn off autorange, null, and auto zero settings.
            * We also set the the DMM to its fastest reading rate, examples using NPLC or resolution are shown.
            * Finally, we optimize the throughput from DMM to PC rate
                * */

            //For more information on getting started using VISA COM I/O operations see the app note located at:
            //http://cp.literature.agilent.com/litweb/pdf/5989-6338EN.pdf

            Ivi.Visa.Interop.ResourceManager rm = new Ivi.Visa.Interop.ResourceManager(); //Open up a new resource manager
            Ivi.Visa.Interop.FormattedIO488 myDmm = new Ivi.Visa.Interop.FormattedIO488(); //Open a new Formatted IO 488 session 
            try
            {
                //string DutAddr = "GPIB0::22"; //String for GPIB
                //string DutAddr = ""TCPIP0::169.254.4.61";  //Example string for LAN
                //string DutAddr = "USB0::0x0957::0x1A07::MY53000101"; //Example string for USB
                string DutAddr = "USB0::0x0957::0x1C07::US00000069::0::INSTR"; //String for GPIB

                myDmm.IO = (IMessage)rm.Open(DutAddr, AccessMode.NO_LOCK, 5000, ""); //Open up a handle to the DMM with a 5 second timeout


                //First start with a reset state
                myDmm.IO.Clear(); //Send a device clear first to stop any measurements in process
                myDmm.WriteString("*RST", true); //Reset the device
                myDmm.WriteString("*IDN?", true); //Get the IDN string                
                string IDN = myDmm.ReadString();
                Console.WriteLine(IDN); //report the DMM's identity


                //Configure for DCV 1V range, 1e-4V resolution for fastest reading on 34461A
                //Setting the range will turn off autorange. Autorange takes a quick reading to determine the range for the DMM. Turning it off will speed up
                //the measurement to you
                myDmm.WriteString("CONF:VOLT:DC 1, 1e-4", true);

                //You can also set for minimum NPLC for fastest reading

                /*1. Setting the NPLC to 100 will give the DMM the most time to integrate the readings going into it. NPLC = Number of power line cycles. 
                 *1 NPLC = 1 power line cycle. Example 1 NPLC in US = 1/60Hz = 16.66 ms. 100NPLC = 1.6 seconds. 
                 *Using an NPLC setting has the added benefit of rejecting power line noise i.e. normal mode rejection. 
                 *Setting the minimum NPLC ensures the minimum resolution without needing to calculate or lookup resolution in measurement units i.e. V, I, Ohms
                */
                myDmm.WriteString("VOLT:DC:NPLC 0.02", true); //Set to 100 NPLC, NPLC can be set to 0.02|0.2|1|10|100        
                double SglRdgTime = 0.02 * (1 / 60); //Calculate the single reading time assuming using a 60Hz power line frequency
                int SglRdgTimems = (int)Math.Round(SglRdgTime * 1000); //Convert the time to milliseconds and to an integer 

                //Turn off AutoZero so that the DMM is not doing any internal correction reading which will slow down the measurement given to you
                myDmm.WriteString("VOLT:DC:ZERO:AUTO OFF", true); //There is a similar command for most DC functions including Temperature, Current, Resistance

                //Turn off Null
                myDmm.WriteString("VOLT:DC:NULL:STAT OFF", true); //There is a similar command for most functions including Temperature, Current, Resistance

                //The above commands will configure the fastest readings for the DMM. We can also optimize the data transfer rates to the PC by taking our set of readings 
                //then transfer after the end of the readings.


                /* Here we will set the DMM to take multiple readings*/
                //Setup to take 100 readings
                myDmm.WriteString("SAMP:COUN 100", true); //Take 100 readings per sample
                myDmm.IO.Timeout = 100 * SglRdgTimems + 500; //Calculate the timeout for 100 readings, add in a small buffer.
                                
                //myDmm.WriteString("READ?"); //Trigger the DMM to start taking the readings. Turns out that READ? has a little bit of overhead. 
                                              //We can optimize that overhead by using a bus trigger and remove individual readings.

                myDmm.WriteString("TRIG:SOUR BUS", true); //Set the trigger system to bus trigger
                myDmm.WriteString("INIT", true); //Get the DMM into Wait for trigger state
                myDmm.WriteString("*TRG", true); //Trigger the DMM to make measurements
                myDmm.WriteString("DATA:REM? 100, WAIT", true); //Get the readings and wait until 100 reading are available in the reading buffer
                Object[] DCVResult = (object[])myDmm.ReadList(IEEEASCIIType.ASCIIType_Any,",;"); //Get the readings in an array

                foreach (double rdg in DCVResult) //Write out each result to the console
                    Console.WriteLine(rdg.ToString());
                DmmClass.CheckDMMError(myDmm); //Check if the DMM has any errors                
                               
            }
            catch (Exception e)
            {
                Console.WriteLine("Error occured: " + e.Message);
            }
            finally
            {
                //Close out your resources
                try { myDmm.IO.Close(); }
                catch { }
                try { System.Runtime.InteropServices.Marshal.ReleaseComObject(myDmm); }
                catch { }
                try
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(rm);
                }
                catch { }
                Console.WriteLine("Press any key to continue...");
                Console.ReadKey();

            }
        }
            /// <summary>
        /// Checks the error queue and throws an exception if error
        /// </summary>
        /// <param name="myDmm"></param>
        public void CheckDMMError(FormattedIO488 myDmm)
        {

            myDmm.WriteString("SYST:ERR?", true);
            string errStr = myDmm.ReadString();

            if (errStr.Contains("No error")) //If no error, then return
                return;
            //If there is an error, read out all of the errors and return them in an exception
            else
            {
                string errStr2 = "";
                do
                {
                    myDmm.WriteString("SYST:ERR?", true);
                    errStr2 = myDmm.ReadString();
                    if (!errStr2.Contains("No error")) errStr = errStr + "\n" + errStr2;

                } while (!errStr2.Contains("No error"));
                throw new Exception("Exception: Encountered system error(s)\n" + errStr);
            }

        }
        
    }
}
